@@ -32,9 +32,9 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #include "timeline2/view/timelinewidget.h"
 #include <mlt++/MltRepository.h>
 
-#include "utils/KMessageBox_KdenliveCompat.h"
 #include <KIO/OpenFileManagerWindowJob>
 #include <KMessageBox>
+
 #include <QCoreApplication>
 #include <QDesktopServices>
 #include <QDir>
@@ -47,7 +47,7 @@ SPDX-License-Identifier: GPL-3.0-only OR LicenseRef-KDE-Accepted-GPL
 #endif
 
 std::unique_ptr<Core> Core::m_self;
-Core::Core(const QString &packageType)
+Core::Core(LinuxPackageType packageType)
     : audioThumbCache(QStringLiteral("audioCache"), 2000000)
     , taskManager(this)
     , m_packageType(packageType)
@@ -77,7 +77,7 @@ void Core::finishShutdown()
 
 Core::~Core() {}
 
-bool Core::build(const QString &packageType, bool testMode)
+bool Core::build(LinuxPackageType packageType, bool testMode)
 {
     if (m_self) {
         return true;
@@ -90,6 +90,7 @@ bool Core::build(const QString &packageType, bool testMode)
     qRegisterMetaType<QList<QAction *>>("QList<QAction*>");
     qRegisterMetaType<MessageType>("MessageType");
     qRegisterMetaType<stringMap>("stringMap");
+    qRegisterMetaType<sequenceMap>("sequenceMap");
     qRegisterMetaType<audioByteArray>("audioByteArray");
     qRegisterMetaType<QList<ItemInfo>>("QList<ItemInfo>");
     qRegisterMetaType<std::shared_ptr<Mlt::Producer>>("std::shared_ptr<Mlt::Producer>");
@@ -98,12 +99,11 @@ bool Core::build(const QString &packageType, bool testMode)
     qRegisterMetaType<requestClipInfo>("requestClipInfo");
     qRegisterMetaType<QVector<QPair<QString, QVariant>>>("paramVector");
     qRegisterMetaType<ProfileParam *>("ProfileParam*");
+    qRegisterMetaType<ObjectId>("ObjectId");
     KeyframeModel::initKeyframeTypes();
 
-#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
     // Increase memory limit allowed per image
     QImageReader::setAllocationLimit(1024);
-#endif
 
     if (!testMode) {
         // Check if we had a crash
@@ -146,21 +146,31 @@ void Core::initHeadless(const QUrl &url)
     MltConnection::construct(QString());
     m_projectManager = new ProjectManager(this);
     QMetaObject::invokeMethod(pCore->projectManager(), "slotLoadHeadless", Qt::QueuedConnection, Q_ARG(QUrl, url));
+    connect(this, &Core::displayBinMessage, this,
+            [](QString text, int, QList<QAction *>, bool, BinMessage::BinCategory) { qInfo() << QStringLiteral("Bin message: ") << text; });
+    connect(this, &Core::displayBinLogMessage, this, [](QString text, int, QString) { qInfo() << QStringLiteral("Bin message: ") << text; });
 }
 
-void Core::initGUI(bool inSandbox, const QString &MltPath, const QUrl &Url, const QString &clipsToLoad)
+void Core::initGUI(const QString &MltPath, const QUrl &Url, const QString &clipsToLoad)
 {
     m_mainWindow = new MainWindow();
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
 
-    QStringList styles = QQuickStyle::availableStyles();
-    if (styles.contains(QLatin1String("org.kde.desktop"))) {
-        QQuickStyle::setStyle("org.kde.desktop");
-    } else if (styles.contains(QLatin1String("Fusion"))) {
-        QQuickStyle::setStyle("Fusion");
+    // The MLT Factory will be initiated there, all MLT classes will be usable only after this
+    bool inSandbox = m_packageType == LinuxPackageType::AppImage || m_packageType == LinuxPackageType::Flatpak || m_packageType == LinuxPackageType::Snap;
+    if (inSandbox) {
+        // In a sandbox environment we need to search some paths recursively
+        QString appPath = qApp->applicationDirPath();
+        KdenliveSettings::setFfmpegpath(QDir::cleanPath(appPath + QStringLiteral("/ffmpeg")));
+        KdenliveSettings::setFfplaypath(QDir::cleanPath(appPath + QStringLiteral("/ffplay")));
+        KdenliveSettings::setFfprobepath(QDir::cleanPath(appPath + QStringLiteral("/ffprobe")));
+        KdenliveSettings::setMeltpath(QDir::cleanPath(appPath + QStringLiteral("/melt")));
+        MltConnection::construct(QDir::cleanPath(appPath + QStringLiteral("/../share/mlt/profiles")));
+    } else {
+        // Open connection with Mlt
+        MltConnection::construct(MltPath);
     }
-    // ELSE Qt6 see: https://doc.qt.io/qt-6/qtquickcontrols-changes-qt6.html#custom-styles-are-now-proper-qml-modules
-#endif
+
+    // TODO Qt6 see: https://doc.qt.io/qt-6/qtquickcontrols-changes-qt6.html#custom-styles-are-now-proper-qml-modules
 
     connect(this, &Core::showConfigDialog, m_mainWindow, &MainWindow::slotShowPreferencePage);
 
@@ -168,12 +178,6 @@ void Core::initGUI(bool inSandbox, const QString &MltPath, const QUrl &Url, cons
     connect(bin, &Bin::requestShowClipProperties, bin, &Bin::showClipProperties);
     m_mainWindow->addBin(bin, QString(), false);
 
-    // Secondary bins
-    for (int i = 1; i < KdenliveSettings::binsCount(); i++) {
-        bin = new Bin(m_projectItemModel, m_mainWindow, false);
-        m_mainWindow->addBin(bin, QString(), false);
-    }
-
     connect(m_projectItemModel.get(), &ProjectItemModel::refreshPanel, m_mainWindow->activeBin(), &Bin::refreshPanel);
     connect(m_projectItemModel.get(), &ProjectItemModel::refreshClip, m_mainWindow->activeBin(), &Bin::refreshClip);
     connect(m_projectItemModel.get(), &ProjectItemModel::itemDropped, m_mainWindow->activeBin(), &Bin::slotItemDropped, Qt::QueuedConnection);
@@ -185,20 +189,14 @@ void Core::initGUI(bool inSandbox, const QString &MltPath, const QUrl &Url, cons
 
     m_monitorManager = new MonitorManager(this);
     projectManager()->init(Url, clipsToLoad);
+    m_mainWindow->init();
 
-    // The MLT Factory will be initiated there, all MLT classes will be usable only after this
-    if (inSandbox) {
-        // In a sandbox environment we need to search some paths recursively
-        QString appPath = qApp->applicationDirPath();
-        KdenliveSettings::setFfmpegpath(QDir::cleanPath(appPath + QStringLiteral("/ffmpeg")));
-        KdenliveSettings::setFfplaypath(QDir::cleanPath(appPath + QStringLiteral("/ffplay")));
-        KdenliveSettings::setFfprobepath(QDir::cleanPath(appPath + QStringLiteral("/ffprobe")));
-        KdenliveSettings::setMeltpath(QDir::cleanPath(appPath + QStringLiteral("/melt")));
-        m_mainWindow->init(QDir::cleanPath(appPath + QStringLiteral("/../share/mlt/profiles")));
-    } else {
-        // Open connection with Mlt
-        m_mainWindow->init(MltPath);
+    // Secondary bins
+    for (int i = 1; i < KdenliveSettings::binsCount(); i++) {
+        bin = new Bin(m_projectItemModel, m_mainWindow, false);
+        m_mainWindow->addBin(bin, QString(), false);
     }
+
     m_projectItemModel->buildPlaylist(QUuid());
     // load the profiles from disk
     ProfileRepository::get()->refresh();
@@ -220,7 +218,7 @@ void Core::initGUI(bool inSandbox, const QString &MltPath, const QUrl &Url, cons
         // we get the list of profiles
         QVector<QPair<QString, QString>> all_profiles = ProfileRepository::get()->getAllProfiles();
         QStringList all_descriptions;
-        for (const auto &profile : qAsConst(all_profiles)) {
+        for (const auto &profile : std::as_const(all_profiles)) {
             all_descriptions << profile.first;
         }
 
@@ -229,7 +227,7 @@ void Core::initGUI(bool inSandbox, const QString &MltPath, const QUrl &Url, cons
         QString item = QInputDialog::getItem(m_mainWindow, i18nc("@title:window", "Select Default Profile"), i18n("Profile:"), all_descriptions, 0, false, &ok);
         if (ok) {
             ok = false;
-            for (const auto &profile : qAsConst(all_profiles)) {
+            for (const auto &profile : std::as_const(all_profiles)) {
                 if (profile.first == item) {
                     m_profile = profile.second;
                     ok = true;
@@ -261,6 +259,9 @@ void Core::initGUI(bool inSandbox, const QString &MltPath, const QUrl &Url, cons
     if (!Url.isEmpty()) {
         Q_EMIT loadingMessageNewStage(i18n("Loading project…"));
     }
+    connect(this, &Core::displayBinMessage, this, &Core::displayBinMessagePrivate);
+    connect(this, &Core::displayBinLogMessage, this, &Core::displayBinLogMessagePrivate);
+
     QMetaObject::invokeMethod(pCore->projectManager(), "slotLoadOnOpen", Qt::QueuedConnection);
 }
 
@@ -338,7 +339,7 @@ QString Core::openExternalApp(QString appPath, QStringList args)
     process.setProgram(appPath);
 #endif
     process.setArguments(args);
-    if (pCore->packageType() == QStringLiteral("appimage")) {
+    if (pCore->packageType() == LinuxPackageType::AppImage) {
         // Strip appimage custom LD_LIBRARY_PATH...
         QProcessEnvironment env = QProcessEnvironment::systemEnvironment();
         qDebug() << "::: GOT ENV: " << env.value("LD_LIBRARY_PATH") << ", PATH: " << env.value("PATH");
@@ -574,6 +575,11 @@ Mlt::Profile &Core::getProjectProfile()
     return m_projectProfile;
 }
 
+std::pair<int, int> Core::getProjectFpsInfo() const
+{
+    return {m_projectProfile.frame_rate_num(), m_projectProfile.frame_rate_den()};
+}
+
 void Core::updateMonitorProfile()
 {
     m_monitorProfile.set_colorspace(m_projectProfile.colorspace());
@@ -643,9 +649,7 @@ void Core::checkProfileValidity()
     int offset = (getProjectProfile().width() % 2) + (getProjectProfile().height() % 2);
     if (offset > 0) {
         // Profile is broken, warn user
-        if (m_mainWindow->getBin()) {
-            Q_EMIT m_mainWindow->getBin()->displayBinMessage(i18n("Your project profile is invalid, rendering might fail."), KMessageWidget::Warning);
-        }
+        Q_EMIT displayBinMessage(i18n("Your project profile is invalid, rendering might fail."), KMessageWidget::Warning);
     }
 }
 
@@ -664,12 +668,12 @@ double Core::getCurrentFps() const
     return getCurrentProfile()->fps();
 }
 
-QSize Core::getCurrentFrameDisplaySize() const
+const QSize Core::getCurrentFrameDisplaySize() const
 {
     return {qRound(getCurrentProfile()->height() * getCurrentDar()), getCurrentProfile()->height()};
 }
 
-QSize Core::getCurrentFrameSize() const
+const QSize Core::getCurrentFrameSize() const
 {
     return {getCurrentProfile()->width(), getCurrentProfile()->height()};
 }
@@ -765,7 +769,7 @@ int Core::getItemIn(const ObjectId &id)
     return 0;
 }
 
-PlaylistState::ClipState Core::getItemState(const ObjectId &id)
+std::pair<PlaylistState::ClipState, ClipType::ProducerType> Core::getItemState(const ObjectId &id)
 {
     switch (id.type) {
     case KdenliveObjectType::TimelineClip:
@@ -776,19 +780,21 @@ PlaylistState::ClipState Core::getItemState(const ObjectId &id)
         }
         break;
     case KdenliveObjectType::TimelineComposition:
-        return PlaylistState::VideoOnly;
+        return {PlaylistState::VideoOnly, ClipType::Unknown};
     case KdenliveObjectType::BinClip:
-        if (!m_guiConstructed) return PlaylistState::Disabled;
+        if (!m_guiConstructed) {
+            return {PlaylistState::Disabled, ClipType::Unknown};
+        }
         return m_mainWindow->getBin()->getClipState(id.itemId);
     case KdenliveObjectType::TimelineTrack:
-        return currentDoc()->getTimeline(id.uuid)->isAudioTrack(id.itemId) ? PlaylistState::AudioOnly : PlaylistState::VideoOnly;
+        return {currentDoc()->getTimeline(id.uuid)->isAudioTrack(id.itemId) ? PlaylistState::AudioOnly : PlaylistState::VideoOnly, ClipType::Unknown};
     case KdenliveObjectType::Master:
-        return PlaylistState::Disabled;
+        return {PlaylistState::Disabled, ClipType::Unknown};
     default:
         qWarning() << "unhandled object type";
         break;
     }
-    return PlaylistState::Disabled;
+    return {PlaylistState::Disabled, ClipType::Unknown};
 }
 
 int Core::getItemDuration(const ObjectId &id)
@@ -862,6 +868,27 @@ int Core::getItemTrack(const ObjectId &id)
     return 0;
 }
 
+bool Core::itemContainsPos(const ObjectId &id, int pos)
+{
+    if (!m_guiConstructed || (!id.uuid.isNull() && !m_mainWindow->getTimeline(id.uuid))) return false;
+    switch (id.type) {
+    case KdenliveObjectType::TimelineClip:
+    case KdenliveObjectType::TimelineComposition:
+    case KdenliveObjectType::TimelineMix: {
+        int itemPos = getItemPosition(id);
+        if (pos < itemPos) {
+            return false;
+        }
+        if (pos >= itemPos + getItemDuration(id)) {
+            return false;
+        }
+        return true;
+    }
+    default:
+        return true;
+    }
+}
+
 void Core::refreshProjectItem(const ObjectId &id)
 {
     if (!m_guiConstructed || (!id.uuid.isNull() && !m_mainWindow->getTimeline(id.uuid))) return;
@@ -893,7 +920,19 @@ void Core::refreshProjectItem(const ObjectId &id)
         }
         break;
     case KdenliveObjectType::Master:
-        refreshProjectMonitorOnce();
+        if (m_monitorManager->isActive(Kdenlive::ProjectMonitor)) {
+            if (m_monitorManager->projectMonitorVisible()) {
+                m_monitorManager->activateMonitor(Kdenlive::ProjectMonitor, false);
+                m_monitorManager->refreshProjectMonitor(true);
+            }
+            m_monitorManager->markMonitorDirty(Kdenlive::ClipMonitor, id.uuid);
+        } else {
+            m_monitorManager->markMonitorDirty(Kdenlive::ProjectMonitor, id.uuid);
+            if (m_monitorManager->clipMonitorVisible()) {
+                m_monitorManager->activateMonitor(Kdenlive::ClipMonitor, false);
+                m_monitorManager->refreshClipMonitor(true);
+            }
+        }
         break;
     default:
         qWarning() << "unhandled object type";
@@ -977,12 +1016,14 @@ void Core::loadingClips(int count, bool allowInterrupt)
     Q_EMIT m_mainWindow->displayProgressMessage(i18n("Loading clips"), MessageType::ProcessingJobMessage, count, allowInterrupt);
 }
 
-void Core::displayBinMessage(const QString &text, int type, const QList<QAction *> &actions, bool showClose, BinMessage::BinCategory messageCategory)
+void Core::displayBinMessagePrivate(const QString &text, int type, const QList<QAction *> &actions, bool showClose, BinMessage::BinCategory messageCategory)
 {
-    m_mainWindow->getBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), actions, showClose, messageCategory);
+    if (m_mainWindow) {
+        m_mainWindow->getBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), actions, showClose, messageCategory);
+    }
 }
 
-void Core::displayBinLogMessage(const QString &text, int type, const QString logInfo)
+void Core::displayBinLogMessagePrivate(const QString &text, int type, const QString logInfo)
 {
     m_mainWindow->getBin()->doDisplayMessage(text, KMessageWidget::MessageType(type), logInfo);
 }
@@ -1045,7 +1086,7 @@ std::shared_ptr<ProjectItemModel> Core::projectItemModel()
 void Core::invalidateRange(QPair<int, int> range)
 {
     if (!m_guiConstructed || currentDoc()->isBusy() || !m_mainWindow->getCurrentTimeline() || m_mainWindow->getCurrentTimeline()->loading) return;
-    m_mainWindow->getCurrentTimeline()->model()->invalidateZone(range.first, range.second);
+    Q_EMIT m_mainWindow->getCurrentTimeline()->model()->invalidateZone(range.first, range.second);
 }
 
 void Core::invalidateItem(ObjectId itemId)
@@ -1069,7 +1110,7 @@ void Core::invalidateItem(ObjectId itemId)
         break;
     case KdenliveObjectType::Master:
         if (tl) {
-            tl->model()->invalidateZone(0, -1);
+            Q_EMIT tl->model()->invalidateZone(0, -1);
         }
         break;
     default:
@@ -1097,14 +1138,27 @@ void Core::updateItemKeyframes(ObjectId id)
     }
 }
 
-void Core::updateItemModel(ObjectId id, const QString &service)
+void Core::updateItemModel(ObjectId id, const QString &service, const QString &updatedParam)
 {
     if (m_guiConstructed && id.type == KdenliveObjectType::TimelineClip && !m_mainWindow->getCurrentTimeline()->loading &&
         service.startsWith(QLatin1String("fade"))) {
         auto tl = m_mainWindow->getTimeline(id.uuid);
         if (tl) {
             bool startFade = service.startsWith(QLatin1String("fadein")) || service.startsWith(QLatin1String("fade_from_"));
-            tl->controller()->updateClip(id.itemId, {startFade ? TimelineModel::FadeInRole : TimelineModel::FadeOutRole});
+            QVector<int> roles;
+            if (startFade) {
+                roles = {TimelineModel::FadeInRole, TimelineModel::FadeInMethodRole};
+                if (updatedParam == QLatin1String("alpha") || updatedParam == QLatin1String("level")) {
+                    roles << TimelineModel::FadeInMethodRole;
+                }
+            } else {
+                roles = {TimelineModel::FadeOutRole, TimelineModel::FadeOutMethodRole};
+                if (updatedParam == QLatin1String("alpha") || updatedParam == QLatin1String("level")) {
+                    roles << TimelineModel::FadeOutMethodRole;
+                }
+            }
+            qDebug() << "==== UPDATING FADE ROLES";
+            tl->controller()->updateClip(id.itemId, roles);
         }
     }
 }
@@ -1257,9 +1311,9 @@ void Core::switchCapture()
     Q_EMIT recordAudio(-1, !isMediaCapturing());
 }
 
-MediaCapture *Core::getAudioDevice()
+std::shared_ptr<MediaCapture> Core::getAudioDevice()
 {
-    return m_capture.get();
+    return m_capture;
 }
 
 void Core::resetAudioMonitoring()
@@ -1275,14 +1329,6 @@ void Core::setAudioMonitoring(bool enable)
     m_capture->switchMonitorState(enable);
 }
 
-QString Core::getProjectFolderName()
-{
-    if (currentDoc()) {
-        return currentDoc()->projectDataFolder(QStringLiteral()) + QDir::separator();
-    }
-    return QString();
-}
-
 QString Core::getProjectCaptureFolderName()
 {
     if (currentDoc()) {
@@ -1291,7 +1337,7 @@ QString Core::getProjectCaptureFolderName()
     return QString();
 }
 
-QString Core::getTimelineClipBinId(ObjectId id)
+const QString Core::getTimelineClipBinId(ObjectId id)
 {
     if (!m_guiConstructed) {
         return QString();
@@ -1318,9 +1364,9 @@ int Core::getDurationFromString(const QString &time)
     return m_timecode.getFrameCount(time);
 }
 
-void Core::processInvalidFilter(const QString &service, const QString &id, const QString &message)
+void Core::processInvalidFilter(const QString &service, const QString &message, const QString &log)
 {
-    if (m_guiConstructed) Q_EMIT m_mainWindow->assetPanelWarning(service, id, message);
+    if (m_guiConstructed) Q_EMIT m_mainWindow->assetPanelWarning(service, message, log);
 }
 
 void Core::updateProjectTags(int previousCount, const QMap<int, QStringList> &tags)
@@ -1328,9 +1374,9 @@ void Core::updateProjectTags(int previousCount, const QMap<int, QStringList> &ta
     if (previousCount > tags.size()) {
         // Clear previous tags
         for (int i = 1; i <= previousCount; i++) {
-            QString current = currentDoc()->getDocumentProperty(QString("tag%1").arg(i));
+            QString current = currentDoc()->getDocumentProperty(QStringLiteral("tag%1").arg(i));
             if (!current.isEmpty()) {
-                currentDoc()->setDocumentProperty(QString("tag%1").arg(i), QString());
+                currentDoc()->setDocumentProperty(QStringLiteral("tag%1").arg(i), QString());
             }
         }
     }
@@ -1338,7 +1384,7 @@ void Core::updateProjectTags(int previousCount, const QMap<int, QStringList> &ta
     int i = 1;
     while (j.hasNext()) {
         j.next();
-        currentDoc()->setDocumentProperty(QString("tag%1").arg(i), QString("%1:%2").arg(j.value().at(1), j.value().at(2)));
+        currentDoc()->setDocumentProperty(QStringLiteral("tag%1").arg(i), QStringLiteral("%1:%2").arg(j.value().at(1), j.value().at(2)));
         i++;
     }
 }
@@ -1346,8 +1392,8 @@ void Core::updateProjectTags(int previousCount, const QMap<int, QStringList> &ta
 std::unique_ptr<Mlt::Producer> Core::getMasterProducerInstance()
 {
     if (m_guiConstructed && m_mainWindow->getCurrentTimeline()) {
-        std::unique_ptr<Mlt::Producer> producer(
-            m_mainWindow->getCurrentTimeline()->controller()->tractor()->cut(0, m_mainWindow->getCurrentTimeline()->controller()->duration() - 1));
+        const QString scene = m_projectManager->projectSceneList(QString(), true).first;
+        std::unique_ptr<Mlt::Producer> producer(new Mlt::Producer(pCore->getProjectProfile(), "xml-string", scene.toUtf8().constData()));
         return producer;
     }
     return nullptr;
@@ -1383,14 +1429,23 @@ int Core::audioChannels()
     return 2;
 }
 
-void Core::addGuides(const QList<int> &guides)
+void Core::addGuides(const QMap<QUuid, QList<int>> &guides)
 {
-    QMap<GenTime, QString> markers;
-    for (int pos : guides) {
-        GenTime p(pos, pCore->getCurrentFps());
-        markers.insert(p, pCore->currentDoc()->timecode().getDisplayTimecode(p, false));
+    QMapIterator<QUuid, QList<int>> i(guides);
+    while (i.hasNext()) {
+        i.next();
+        QMap<GenTime, QString> markers;
+        for (int pos : i.value()) {
+            GenTime p(pos, pCore->getCurrentFps());
+            markers.insert(p, pCore->currentDoc()->timecode().getDisplayTimecode(p, false));
+        }
+        auto timeline = m_mainWindow->getTimeline(i.key());
+        if (timeline == nullptr) {
+            // Timeline not found, default to active one
+            timeline = m_mainWindow->getCurrentTimeline();
+        }
+        timeline->controller()->getModel()->getGuideModel()->addMarkers(markers);
     }
-    m_mainWindow->getCurrentTimeline()->controller()->getModel()->getGuideModel()->addMarkers(markers);
 }
 
 void Core::temporaryUnplug(const QList<int> &clipIds, bool hide)
@@ -1419,6 +1474,11 @@ void Core::showEffectZone(ObjectId id, QPair<int, int> inOut, bool checked)
     if (m_guiConstructed && id.type != KdenliveObjectType::BinClip) {
         auto tl = m_mainWindow->getTimeline(id.uuid);
         if (tl) {
+            if (id.type == KdenliveObjectType::TimelineClip) {
+                int offset = getItemPosition(id);
+                inOut.first += offset;
+                inOut.second += offset;
+            }
             tl->controller()->showRulerEffectZone(inOut, checked);
         }
     }
@@ -1476,13 +1536,6 @@ void Core::cleanup()
     }
 }
 
-#if KNEWSTUFF_VERSION < QT_VERSION_CHECK(5, 98, 0)
-int Core::getNewStuff(const QString &config)
-{
-    return m_mainWindow->getNewStuff(config);
-}
-#endif
-
 void Core::addBin(const QString &id)
 {
     Bin *bin = new Bin(m_projectItemModel, m_mainWindow, false);
@@ -1517,6 +1570,28 @@ int Core::getAssetGroupedInstance(const ObjectId &id, const QString &assetId)
     }
 }
 
+QList<std::shared_ptr<KeyframeModelList>> Core::getGroupKeyframeModels(const ObjectId &id, const QString &assetId)
+{
+    if (KdenliveSettings::applyEffectParamsToGroup()) {
+        switch (id.type) {
+        case KdenliveObjectType::TimelineClip:
+            if (auto tl = currentDoc()->getTimeline(id.uuid)) {
+                return tl->getGroupKeyframeModels(id.itemId, assetId);
+            }
+            break;
+        case KdenliveObjectType::BinClip:
+            if (bin() != nullptr) {
+                return bin()->getGroupKeyframeModels(id.itemId, assetId);
+            }
+            break;
+        default:
+            // Nothing to do
+            break;
+        }
+    }
+    return {};
+}
+
 void Core::groupAssetCommand(const ObjectId &id, const QString &assetId, const QModelIndex &index, const QString &previousValue, QString value,
                              QUndoCommand *command)
 {
@@ -1636,7 +1711,7 @@ void Core::highlightFileInExplorer(QList<QUrl> urls)
     if (urls.isEmpty()) {
         return;
     }
-    if (m_packageType == QLatin1String("flatpak")) {
+    if (m_packageType == LinuxPackageType::Flatpak) {
         QDesktopServices::openUrl(QUrl::fromLocalFile(QFileInfo(urls.first().toLocalFile()).absolutePath()));
     } else {
         KIO::highlightInFileManager(urls);
@@ -1649,3 +1724,75 @@ void Core::updateHoverItem(const QUuid &uuid)
         m_mainWindow->getCurrentTimeline()->regainFocus();
     }
 }
+
+std::pair<bool, bool> Core::assetHasAV(ObjectId id)
+{
+    switch (id.type) {
+    case KdenliveObjectType::Master:
+        return {true, true};
+    case KdenliveObjectType::TimelineTrack: {
+        auto timeline = m_mainWindow->getTimeline(id.uuid);
+        if (timeline && timeline->model()->isAudioTrack(id.itemId)) {
+            return {true, false};
+        }
+        return {false, true};
+    }
+    case KdenliveObjectType::TimelineClip: {
+        auto timeline = m_mainWindow->getTimeline(id.uuid);
+        if (timeline && timeline->model()->clipIsAudio(id.itemId)) {
+            return {true, false};
+        }
+        return {false, true};
+    }
+    case KdenliveObjectType::BinClip: {
+        PlaylistState::ClipState state = bin()->getClipState(id.itemId).first;
+        if (state == PlaylistState::Disabled) {
+            return {true, true};
+        } else if (state == PlaylistState::AudioOnly) {
+            return {true, false};
+        } else {
+            return {false, true};
+        }
+    }
+    default:
+        return {false, false};
+    }
+}
+
+void Core::showEffectStackFromId(ObjectId owner)
+{
+    switch (owner.type) {
+    case KdenliveObjectType::BinClip:
+        activeBin()->showItemEffectStack(owner);
+        break;
+    case KdenliveObjectType::TimelineClip:
+        if (m_guiConstructed && m_mainWindow->getCurrentTimeline()->controller()) {
+            m_mainWindow->getCurrentTimeline()->controller()->showAsset(owner.itemId);
+        }
+        break;
+    case KdenliveObjectType::TimelineTrack:
+        if (m_guiConstructed && m_mainWindow->getCurrentTimeline()->controller()) {
+            m_mainWindow->getCurrentTimeline()->controller()->showTrackAsset(owner.itemId);
+        }
+        break;
+    case KdenliveObjectType::Master:
+        if (m_guiConstructed && m_mainWindow->getCurrentTimeline()->controller()) {
+            m_mainWindow->getCurrentTimeline()->controller()->showMasterEffects();
+        }
+        break;
+    default:
+        break;
+    }
+}
+
+void Core::openDocumentationLink(const QUrl &link)
+{
+    if (KMessageBox::questionTwoActions(
+            QApplication::activeWindow(),
+            i18n("This will open a browser to display Kdenlive's online documentation at the following url:\n %1", link.toDisplayString()), {},
+            KGuiItem(i18n("Open Browser")), KStandardGuiItem::cancel(), QStringLiteral("allow_browser_help")) == KMessageBox::SecondaryAction) {
+        // Stop
+        return;
+    }
+    QDesktopServices::openUrl(link);
+}
