@@ -193,7 +193,7 @@ void TaskManager::taskDone(int cid, AbstractTask *task)
         return;
     }
     m_tasksListLock.lockForWrite();
-    if (m_taskList.find(cid) != m_taskList.end()) {
+    if (!m_taskList.empty() && m_taskList.find(cid) != m_taskList.end()) {
         m_taskList[cid].erase(std::remove(m_taskList[cid].begin(), m_taskList[cid].end(), task), m_taskList[cid].end());
         if (m_taskList[cid].size() == 0) {
             m_taskList.erase(cid);
@@ -217,19 +217,23 @@ void TaskManager::slotCancelJobs(bool leaveBlocked, const QVector<AbstractTask::
     }
     m_tasksListLock.lockForWrite();
     m_blockUpdates = true;
+    qDebug() << "ZZZZZZZZZZZZZZZZZZZZZZZ\n\nSTARTING TASKMANAGER CLOSURE, ACTIVE THREADS: " << m_taskPool.activeThreadCount() << "\nEXCEPTIONS: " << exceptions
+             << "\n\nZZZZZZZZZZZZZZZZZZZZZZZ";
 
     for (const auto &task : m_taskList) {
         int ix = task.second.size() - 1;
         while (ix >= 0) {
             AbstractTask *t = task.second.at(ix);
             AbstractTask::JOBTYPE taskType = t->m_type;
-            if (exceptions.contains(taskType) || t->isCanceled() || t->m_progress == 100) {
+            if (exceptions.contains(taskType)) {
                 ix--;
                 continue;
             }
             if (taskType != AbstractTask::TRANSCODEJOB && taskType != AbstractTask::PROXYJOB) {
                 if (m_taskPool.tryTake(t)) {
                     // Task was not started yet, we can simply delete
+                    qDebug() << "** DELETED  1 TASK from task pool: " << taskType;
+                    m_taskList[task.first].erase(std::remove(m_taskList[task.first].begin(), m_taskList[task.first].end(), t), m_taskList[task.first].end());
                     delete t;
                     ix--;
                     continue;
@@ -237,6 +241,7 @@ void TaskManager::slotCancelJobs(bool leaveBlocked, const QVector<AbstractTask::
             } else {
                 if (m_transcodePool.tryTake(t)) {
                     // Task was not started yet, we can simply delete
+                    m_taskList[task.first].erase(std::remove(m_taskList[task.first].begin(), m_taskList[task.first].end(), t), m_taskList[task.first].end());
                     delete t;
                     ix--;
                     continue;
@@ -244,24 +249,33 @@ void TaskManager::slotCancelJobs(bool leaveBlocked, const QVector<AbstractTask::
             }
             if (m_taskList.find(task.first) != m_taskList.end()) {
                 // If so, then just add ourselves to be notified upon completion.
-                t->cancelJob();
-                t->m_runMutex.lock();
-                t->m_runMutex.unlock();
-                t->deleteLater();
+                if (t->cancelJob()) {
+                    m_taskList[task.first].erase(std::remove(m_taskList[task.first].begin(), m_taskList[task.first].end(), t), m_taskList[task.first].end());
+                    t->m_runMutex.lock();
+                    t->m_runMutex.unlock();
+                    t->deleteLater();
+                }
             }
             ix--;
         }
     }
+    m_tasksListLock.unlock();
     if (exceptions.isEmpty()) {
-        m_taskPool.waitForDone();
-        m_transcodePool.waitForDone();
+        if (!m_taskPool.waitForDone(5000)) {
+            qDebug() << "====== FAILED TO TERMINATE ALL TASKS. Currently alive: " << m_taskPool.activeThreadCount();
+            Q_ASSERT(false);
+        }
+        if (!m_transcodePool.waitForDone(5000)) {
+            qDebug() << "====== FAILED TO TERMINATE ALL TRANSCODE TASKS. Currently alive: " << m_transcodePool.activeThreadCount();
+            Q_ASSERT(false);
+        }
+        QWriteLocker lock(&m_tasksListLock);
         m_taskList.clear();
         m_taskPool.clear();
     }
-    m_tasksListLock.unlock();
-    // Set jobs count
-    Q_EMIT jobCount(0);
     if (!leaveBlocked) {
+        // Set jobs count
+        Q_EMIT jobCount(0);
         m_blockUpdates = false;
     }
 }
@@ -289,7 +303,6 @@ void TaskManager::startTask(int ownerId, AbstractTask *task)
     for (const auto &task : m_taskList) {
         count += task.second.size();
     }
-    m_tasksListLock.unlock();
     // Set jobs count
     Q_EMIT jobCount(count);
     if (task->m_type == AbstractTask::TRANSCODEJOB || task->m_type == AbstractTask::PROXYJOB) {
@@ -298,6 +311,7 @@ void TaskManager::startTask(int ownerId, AbstractTask *task)
     } else {
         m_taskPool.start(task, task->m_priority);
     }
+    m_tasksListLock.unlock();
 }
 
 int TaskManager::getJobProgressForClip(const ObjectId &owner)
