@@ -33,6 +33,7 @@ class ProfileModel;
 class MarkerListModel;
 class MarkerSortModel;
 class PreviewManager;
+class OtioImport;
 
 /** @brief This class represents a Timeline object, as viewed by the backend.
    In general, the Gui associated with it will send modification queries (such as resize or move), and this class authorize them or not depending on the
@@ -98,6 +99,7 @@ public:
     friend class ProjectManager;
     template <typename T> friend class MoveableItem;
     friend class ClipModel;
+    friend class ClipDurationDialog;
     friend class CompositionModel;
     friend class GroupsModel;
     friend class TimelineController;
@@ -105,6 +107,9 @@ public:
     friend class MarkerListModel;
     friend class TimeRemap;
     friend struct TimelineFunctions;
+    friend class KdenliveTests;
+    friend class OtioExport;
+    friend class OtioImport;
 
     bool isClosed{true};
     Q_PROPERTY(QString visibleSequenceName MEMBER m_visibleSequenceName NOTIFY visibleSequenceNameChanged)
@@ -153,6 +158,8 @@ public:
         TrackTagRole,         /// track only
         FadeInRole,           /// clip only
         FadeOutRole,          /// clip only
+        FadeInMethodRole,     /// clip only
+        FadeOutMethodRole,    /// clip only
         FileHashRole,         /// clip only
         SpeedRole,            /// clip only
         ClipThumbRole,        /// clip only
@@ -163,6 +170,7 @@ public:
         ItemIdRole,
         ThumbsFormatRole,   /// track only
         EffectNamesRole,    /// track and clip only
+        EffectCountRole,    /// track and clip only
         EffectsEnabledRole, /// track and clip only
         GrabbedRole,        /// clip+composition only
         SelectedRole,       /// clip+composition only
@@ -211,6 +219,9 @@ public:
        @param clipId Id of the clip to test */
     Q_INVOKABLE int getClipTrackId(int clipId) const;
 
+    /** @brief Returns the row of the effect if the clip has it in its effectstack, -1 otherwise */
+    int clipAssetRow(int cid, const QString &assetId, int eid = -1) const;
+
     /** @brief Returns the id of the track containing composition (-1 if it is not inserted)
        @param clipId Id of the composition to test */
     Q_INVOKABLE int getCompositionTrackId(int compoId) const;
@@ -220,6 +231,7 @@ public:
 
     Q_INVOKABLE int getCompositionPosition(int compoId) const;
     int getSubtitlePosition(int subId) const;
+    int getSubtitleLayer(int subId) const;
     int getCompositionPlaytime(int compoId) const;
     int getCompositionEnd(int compoId) const;
     std::pair<int, int> getMixInOut(int cid) const;
@@ -227,11 +239,14 @@ public:
 
     /** @brief Returns an item position, item can be clip, subtitle or composition */
     Q_INVOKABLE int getItemPosition(int itemId) const;
+    Q_INVOKABLE int getItemFakePosition(int itemId) const;
     /** @brief Returns an item duration, item can be clip, subtitle or composition */
     int getItemPlaytime(int itemId) const;
     /** @brief Returns an item's out point on its track, item can be clip, subtitle or composition */
     int getItemEnd(int itemId) const;
-
+    /** @brief Returns a clip in / out frames */
+    std::pair<int, int> getClipInOut(int cid) const;
+    bool clipIsValid(int cid) const;
     /** @brief Returns the subplaylist index of a clip in a track */
     int getClipSubPlaylistIndex(int cid) const;
     /** @brief Returns the name of a timeline clip */
@@ -268,6 +283,9 @@ public:
     /** @brief Helper function that returns true if the given ID is in a group */
     Q_INVOKABLE bool isInGroup(int id) const;
 
+    /** @brief Returns all basic infos about a clip (position, crop, trackId, ...) */
+    ItemInfo getItemInfo(int id) const;
+
     /** @brief Given a composition Id, returns its underlying parameter model */
     std::shared_ptr<AssetParameterModel> getCompositionParameterModel(int compoId) const;
     /** @brief Given a clip Id, returns its underlying effect stack model */
@@ -308,9 +326,9 @@ public:
     */
     QPoint getClipInDuration(int clipId) const;
 
-    /** @brief Returns the clip state (audio/video only)
+    /** @brief Returns the clip state (audio/video only) and type (Video, Color, Title...)
      */
-    PlaylistState::ClipState getClipState(int clipId) const;
+    std::pair<PlaylistState::ClipState, ClipType::ProducerType> getClipState(int clipId) const;
 
     /** @brief Returns the bin id of the clip master
        @param clipId Id of the clip to test
@@ -369,6 +387,7 @@ public:
        @param trackId Id of the track to test
     */
     int getPreviousVideoTrackIndex(int trackId) const;
+    int getTopVideoTrackIndex();
 
     /** @brief Set the marker model on this timeline (usually the marker model from its Bin Sequence clip.
      */
@@ -401,13 +420,14 @@ public:
     */
     Q_INVOKABLE bool requestClipMove(int clipId, int trackId, int position, bool moveMirrorTracks = true, bool updateView = true, bool logUndo = true,
                                      bool invalidateTimeline = false, bool revertMove = false);
-    Q_INVOKABLE bool requestSubtitleMove(int clipId, int position, bool updateView = true, bool logUndo = true, bool finalMove = false);
-    bool requestSubtitleMove(int clipId, int position, bool updateView, bool first, bool last, bool finalMove, Fun &undo, Fun &redo);
+    Q_INVOKABLE bool requestSubtitleMove(int clipId, int layer, int position, bool updateView = true, bool logUndo = true, bool finalMove = false,
+                                         bool fakeMove = false);
+    bool requestSubtitleMove(int clipId, int layer, int position, bool updateView, bool first, bool last, bool finalMove, Fun &undo, Fun &redo);
     /** @brief return the previous blank frame on a track */
     Q_INVOKABLE int getPreviousBlank(int trackId, int pos);
     /** @brief return the next blank frame on a track */
     Q_INVOKABLE int getNextBlank(int trackId, int pos);
-    int cutSubtitle(int position, Fun &undo, Fun &redo);
+    int cutSubtitle(int layer, int position, Fun &undo, Fun &redo);
     bool requestClipMix(const QString &mixId, std::pair<int, int> clipIds, std::pair<int, int> mixDurations, int trackId, int position, bool updateView,
                         bool invalidateTimeline, bool finalMove, Fun &undo, Fun &redo, bool groupMove);
 
@@ -417,7 +437,7 @@ public:
        the clip is in a group, the call is deferred to requestGroupMove @param
        transid is the ID of the composition @param trackId is the ID of the
        track */
-    Q_INVOKABLE bool requestCompositionMove(int compoId, int trackId, int position, bool updateView = true, bool logUndo = true);
+    Q_INVOKABLE bool requestCompositionMove(int compoId, int trackId, int position, bool updateView = true, bool logUndo = true, bool fakeMove = false);
 
     /* Same function, but accumulates undo and redo, and doesn't check
        for group*/
@@ -444,10 +464,11 @@ public:
        @param dontRefreshMasterClip when false, no view refresh is attempted
        @returns  a list in the form {position, trackId}
         */
-    Q_INVOKABLE QVariantList suggestItemMove(int itemId, int trackId, int position, int cursorPosition, int snapDistance = -1);
-    Q_INVOKABLE QVariantList suggestClipMove(int clipId, int trackId, int position, int cursorPosition, int snapDistance = -1, bool moveMirrorTracks = true);
-    Q_INVOKABLE int suggestSubtitleMove(int subId, int position, int cursorPosition, int snapDistance);
-    Q_INVOKABLE QVariantList suggestCompositionMove(int compoId, int trackId, int position, int cursorPosition, int snapDistance = -1);
+    Q_INVOKABLE QVariantList suggestItemMove(int itemId, int trackId, int position, int cursorPosition, int snapDistance = -1, bool fakeMove = false);
+    Q_INVOKABLE QVariantList suggestClipMove(int clipId, int trackId, int position, int cursorPosition, int snapDistance = -1, bool moveMirrorTracks = true,
+                                             bool fakeMove = false);
+    Q_INVOKABLE int suggestSubtitleMove(int subId, int newLayer, int position, int cursorPosition, int snapDistance, bool fakeMove = false);
+    Q_INVOKABLE QVariantList suggestCompositionMove(int compoId, int trackId, int position, int cursorPosition, int snapDistance = -1, bool fakeMove = false);
     /** @brief returns the frame pos adjusted to edit mode
      */
     Q_INVOKABLE int adjustFrame(int frame, int trackId);
@@ -513,6 +534,11 @@ public:
     /**  @brief True if we are selecting a single item in a group
      */
     bool singleSelectionMode() const;
+    /**  @brief CHange the black bacground track duration.
+     *  if @limit is  true, reduce track duration to project duration
+     *  if @limit is false, add seek offset
+     */
+    void limitBlackTrack(bool limit);
 
 protected:
     /** @brief Creates a new clip instance without inserting it.
@@ -526,6 +552,8 @@ protected:
 
     /** @brief Switch item selection status */
     void setSelected(int itemId, bool sel);
+    /** @brief Check if selection is 2 clips from the same bin clip and check offset */
+    void checkAndUpdateOffset(std::unordered_set<int> pairIds);
 
 public:
     /** @brief Deletes the given clip or composition from the timeline.
@@ -637,7 +665,7 @@ public:
     */
     bool requestClipUngroup(int itemId, bool logUndo = true);
     /** Same function, but accumulates undo and redo @see requestClipUngroup*/
-    bool requestClipUngroup(int itemId, Fun &undo, Fun &redo);
+    bool requestClipUngroup(int itemId, Fun &undo, Fun &redo, bool onDeletion = false);
     /** Remove an item from a group*/
     bool requestRemoveFromGroup(int itemId, Fun &undo, Fun &redo);
     /** @brief convenience functions for several ids at the same time */
@@ -667,6 +695,11 @@ public:
        Returns the duration in frames
     */
     int duration() const;
+    /** @brief Get project duration with 2 values
+       First is the real sequence duration
+       Second is the last used frame in timeline instances, or 0 if that value is smaller than first value.
+    */
+    std::pair<int, int> durations() const;
     static int seekDuration; /// Duration after project end where seeking is allowed
     /** @brief True until the timeline has all tracks and clips loaded
      */
@@ -700,7 +733,8 @@ protected:
        @param snapDistance the maximum distance for a snap result, -1 for no snapping
        @returns best snap position or -1 if no snap point is near
      */
-    int getBestSnapPos(int referencePos, int diff, std::vector<int> pts = std::vector<int>(), int cursorPosition = 0, int snapDistance = -1);
+    int getBestSnapPos(int referencePos, int diff, std::vector<int> pts = std::vector<int>(), int cursorPosition = 0, int snapDistance = -1,
+                       bool fakeMove = false);
 
     /** @brief Returns the best possible size for a clip on resize
      */
@@ -714,7 +748,7 @@ protected:
      *  @returns a pair with { original group id of the element, the id of the other element
      *  in the group if it was a 2 items group and group was deleted in the operation }
      */
-    std::pair<int, int> extractSelectionFromGroup(std::unordered_set<int> selection, Fun &undo, Fun &redo);
+    std::pair<int, int> extractSelectionFromGroup(int selection, Fun &undo, Fun &redo, bool onDeletion = false);
 
 public:
     /** @brief Requests the next snapped point
@@ -773,7 +807,7 @@ public:
 
     /** @brief Get a timeline clip id by its position or -1 if not found
      */
-    int getClipByPosition(int trackId, int position, int playlist = -1) const;
+    int getClipByPosition(int trackId, int position, int playlistOrLayer = -1) const;
     int getClipByStartPosition(int trackId, int position) const;
 
     /** @brief Get a timeline composition id by its starting position or -1 if not found
@@ -781,8 +815,8 @@ public:
     int getCompositionByPosition(int trackId, int position) const;
     /** @brief Get a timeline subtitle id by its starting position or -1 if not found
      */
-    int getSubtitleByStartPosition(int position) const;
-    int getSubtitleByPosition(int position) const;
+    int getSubtitleByStartPosition(int layer, int position) const;
+    int getSubtitleByPosition(int layer, int position) const;
 
     /** @brief Returns a list of all items that are intersect with a given range.
      * @param trackId is the id of the track for concerned items. Setting trackId to -1 returns items on all tracks
@@ -922,12 +956,6 @@ protected:
      */
     void registerComposition(const std::shared_ptr<CompositionModel> &composition);
 
-    void registerSubtitle(int id, GenTime startTime, bool temporary = false);
-    void deregisterSubtitle(int id, bool temporary = false);
-    /** @brief Returns the index for a subtitle's id (it's position in the list
-     */
-    int positionForIndex(int id);
-
     /** @brief Register a new group. This is a call-back meant to be called from GroupsModel
      */
     void registerGroup(int groupId);
@@ -984,8 +1012,8 @@ protected:
     /** @brief Attempt to make a clip move without ever updating the view */
     bool requestClipMoveAttempt(int clipId, int trackId, int position);
 
-    int getSubtitleIndex(int subId) const;
-    std::pair<int, GenTime> getSubtitleIdFromIndex(int index) const;
+    /** @brief Return all subtitle ids */
+    std::unordered_set<int> getAllSubIds();
 
 public:
     /** @brief Debugging function that checks consistency with Mlt objects */
@@ -1016,8 +1044,6 @@ Q_SIGNALS:
     void checkItemDeletion(int cid);
     /** @brief request animation of the track tid lock icon */
     void flashLock(int tid);
-    /** @brief Save guide categories in document properties */
-    void saveGuideCategories();
     /** @brief Highlight a subtitle item in timeline */
     void highlightSub(int index);
     /** @brief The visible sequence name has to be changed */
@@ -1045,9 +1071,6 @@ protected:
     std::unordered_map<int, std::shared_ptr<CompositionModel>>
         m_allCompositions; // the keys are the composition id, and the values are the corresponding pointers
 
-    // TODO: move this in subtitlemodel.h
-    std::map<int, GenTime> m_allSubtitles;
-
     std::unique_ptr<GroupsModel> m_groups;
     std::shared_ptr<SnapModel> m_snaps;
     std::shared_ptr<SubtitleModel> m_subtitleModel{nullptr};
@@ -1094,6 +1117,7 @@ protected:
 protected:
     /** @brief Rebuild track compositing */
     virtual void buildTrackCompositing(bool rebuild = false) = 0;
+    virtual void removeTrackCompositing() = 0;
     virtual void _beginRemoveRows(const QModelIndex &, int, int) = 0;
     virtual void _beginInsertRows(const QModelIndex &, int, int) = 0;
     virtual void _endRemoveRows() = 0;
